import Vue from "nativescript-vue";
import { Frame } from "@nativescript/core/ui/frame";
import { RouterService } from "./router-service";
import { registerActionDispatcher } from "./router-dispatcher-service";
import routerMixin from "./router-mixin";
export const createRouter = (vueRouterOptions, routerOptions) => {
    const vm = routerOptions.vm || Vue.prototype;
    const router = new RouterService(vueRouterOptions, Object.assign({ frame: Frame, vm }, routerOptions));
    if (vm.config && vm.config.globalProperties) {
        vm.config.globalProperties.$routeTo = router.push.bind(router);
        vm.config.globalProperties.$routeBack = router.back.bind(router);
        vm.config.globalProperties.$router = router;
    }
    else {
        vm.$routeTo = router.push.bind(router);
        vm.$routeBack = router.back.bind(router);
        vm.$router = router;
    }
    if (vm.mixin) {
        vm.mixin(routerMixin);
    }
    if (vm.$store) {
        registerActionDispatcher(router, vm.$store);
    }
    return router;
};
export default {
    createRouter,
};
//# sourceMappingURL=vue-router.common.js.map